{% for example in handler.get_content_examples(definition) %}
{% if example.value %}
```json
{{handler.write_content_example(example, content_type) | safe}}
```
{% if example.auto_generated %}_{{texts.auto_generated_example_note | wordwrap(80)}}_{% endif -%}
{% endif %}
{% endfor %}
