{% for tag, operations in handler.get_operations().items() %}
## <span class="api-tag">{{tag or "Endpoints"}}</span>

{% for path, definition in operations %}
{%- for http_method, operation in definition.items() %}
<hr class="operation-separator" />

### <span class="http-{{http_method.lower()}}">{{http_method.upper()}}</span> {{path | route | safe}}
{% if "summary" in operation -%}
{{operation.summary | wordwrap(80)}}
{%- endif -%}

{%- if operation.description and operation.summary != operation.description %}

??? note "{{texts.description.title()}}"
    {{operation.description | wordwrap(76) | indent(4)}}
{% endif %}

{%- with parameters = handler.get_parameters(operation) -%}
{%- if parameters %}
{% include "partial/request-parameters.html" %}
{%- endif -%}
{%- endwith -%}

{%- if operation.requestBody %}
{% include "partial/request-body.html" %}
{%- endif %}

{%- if operation.responses %}
{%- include "partial/request-responses.html" %}
{%- endif %}
{% endfor %}
{% endfor %}
{% endfor %}
